/*
 * do not trap Cxxx, except as needed:
 *   need to read C013, C014, C016 upon every read or write
 *   write to C073 sets our bank, but what to do with an invalid bank?
 *   bank 0 = standard RAM, so don't interfere.
 *   a C073 write does not OVERRIDE the standard effect.
 */
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

typedef struct {
 uint8_t iiemode; /* emulating a //e */
 uint8_t store80,
         rdaux,
         wraux,
         lcaux,
         cxoverride,
         c3override,
         txt80,
         dblgrx,
         mostxt,
         iou,
         an3,
         alt[2];
 uint8_t lcread,
         lcwrite,
         lcalt;
 uint8_t nogx,
         split,
         page2,
         hires;

 uint8_t *ram;
 uint8_t enh;
} IIE;

IIE *iie;

static uint8_t xram[48][65536];
static uint8_t xrambank;

void (*cpuwrite)(uint16_t, uint8_t);
uint8_t (*cpuread)(uint16_t);

__declspec(dllexport) uint32_t mod_type=0x544F4C53;
__declspec(dllexport) uint8_t *mod_name="RAM Works Memory Expansion";
__declspec(dllexport) uint8_t *mod_pref="ramworks";
__declspec(dllexport) uint8_t *mod_vers="0.01";
__declspec(dllexport) uint8_t *mod_copr="Copyright 2003, 2017 Steve Nickolas";

__declspec(dllexport) uint8_t  *ramworks_preload_name=0;
__declspec(dllexport) uint16_t ramworks_preload_size=0;
__declspec(dllexport) uint8_t  *ramworks_preload_ptr=0;

__declspec(dllexport) void ramworks_write16 (uint16_t a, uint8_t v)
{
}

__declspec(dllexport) void ramworks_write256 (uint16_t a, uint8_t v)
{
}

__declspec(dllexport) void ramworks_write2048 (uint16_t a, uint8_t v)
{
}

__declspec(dllexport) uint8_t ramworks_read16 (uint16_t a)
{
 return 255;
}

__declspec(dllexport) uint8_t ramworks_read256 (uint16_t a)
{
 return 255;
}

__declspec(dllexport) uint8_t ramworks_read2048 (uint16_t a)
{
 return 255;
}

__declspec(dllexport) int16_t ramworks_override_wr (uint16_t a, uint8_t v)
{
 if ((a&0xF000)==0xC000)
 {
  if (a==0xC073) xrambank=v;
  
  return -1;
 }
 if (!xrambank) return -1;
 if (((a<0x0200)||(a>=0xD000)&&(iie->lcwrite))&&(!(iie->lcaux))) return -1;
 if (((a>=0x0200)&&(a<0xC000))&&(!(iie->wraux))) return -1;
 if ((a>=0xD000)&&(a<0xE000)&&(iie->lcalt)) a-=0x1000;
 
 //printf ("%02X -> %02X/%04X\n",v,xrambank,a);
 if (xrambank<48) xram[xrambank][a]=v;
 return 0;
}

__declspec(dllexport) int16_t ramworks_override_rd (uint16_t a)
{
 if ((a&0xF000)==0xC000) return -1;
 if (!xrambank) return -1;
 if (((a<0x0200)||(a>=0xD000)&&(iie->lcread))&&(!(iie->lcaux))) return -1;
 if (((a>=0x0200)&&(a<0xC000))&&(!(iie->rdaux))) return -1;
 if ((a>=0xD000)&&(a<0xE000)&&(iie->lcalt)) a-=0x1000;
 
 //printf ("%02X/%04X -> %02X\n",xrambank,a,(xrambank<48)?xram[xrambank][a]:0);
 if (xrambank<48) return xram[xrambank][a];
 return 0;
}

__declspec(dllexport) int ramworks_reset (void)
{
}

__declspec(dllexport) int ramworks_loop (uint32_t cycles)
{
}

__declspec(dllexport) void ramworks_dmainit(void (*v_cpuwrite)(uint16_t, uint8_t),
                                            uint8_t (*v_cpuread)(uint16_t), IIE *my_iie)
{
 cpuwrite=v_cpuwrite;
 cpuread=v_cpuread;
 iie=my_iie;
}

__declspec(dllexport) int ramworks_init (uint8_t slot)
{
 if (slot!=3)
 {
  fprintf (stderr, "RAM Works adapter must be installed in slot 3\n");
  return -1;
 }
 
 for (xrambank=0; xrambank<48; xrambank++) memset(xram[xrambank],0,65536);
 xrambank=0;
 
 return 6502; /* need more detailed info on system arch */
}

__declspec(dllexport) int ramworks_deinit (void)
{
}

__declspec(dllexport) void ramworks_config (void)
{
}

